import click

from prelude_cli.views.shared import handle_api_error
from prelude_sdk.controllers.build_controller import BuildController
from prelude_sdk.controllers.detect_controller import DetectController
from prelude_sdk.models.codes import Colors, RunCode, Lookup

from rich import print_json
from rich.console import Console
from rich.table import Table


@click.group()
@click.pass_context
def detect(ctx):
    """ Continuously test your endpoints """
    ctx.obj = DetectController(account=ctx.obj)


@detect.command('create-endpoint')
@click.option('--tags', help='a comma-separated list of tags for this endpoint', type=str)
@click.argument('name')
@click.pass_obj
@handle_api_error
def register_endpoint(controller, name, tags):
    """ Register a new endpoint """
    endpoint_token = controller.register_endpoint(name=name, tags=tags.split(',') if tags else [])
    click.secho(f'Endpoint token: {endpoint_token}', fg=Colors.GREEN.value)


@detect.command('enable-test')
@click.argument('test')
@click.option('--tags', help='only enable for these tags')
@click.option('--run_code',
              help='provide a run_code',
              default='daily', show_default=True,
              type=click.Choice(['daily', 'weekly', 'monthly', 'once', 'debug'], case_sensitive=False))
@click.pass_obj
@handle_api_error
def activate_test(controller, test, run_code, tags):
    """ Add TEST to your queue """
    tags = tags.split(',') if tags else []
    controller.enable_test(ident=test, run_code=RunCode[run_code.upper()].value, tags=tags)


@detect.command('disable-test')
@click.argument('test')
@click.confirmation_option(prompt='Are you sure?')
@click.pass_obj
@handle_api_error
def deactivate_test(controller, test):
    """ Remove TEST from your queue """
    controller.disable_test(ident=test)
    click.secho(f'Disabled {test}', fg=Colors.GREEN.value)


@detect.command('delete-endpoint')
@click.argument('endpoint_id')
@click.confirmation_option(prompt='Are you sure?')
@click.pass_obj
@handle_api_error
def delete_endpoint(controller, endpoint_id):
    """Delete a probe/endpoint"""
    controller.delete_endpoint(ident=endpoint_id)
    click.secho(f'Deleted {endpoint_id}', fg=Colors.GREEN.value)


@detect.command('list-queue')
@click.pass_obj
@handle_api_error
def queue(controller):
    """ List all tests in your active queue """
    build = BuildController(account=controller.account)
    tests = {row['id']: row['rule'] for row in build.list_tests()}
    active = controller.print_queue()
    for q in active:
        q['run_code'] = RunCode(q['run_code']).name
        q['rule'] = tests[q['test']]
    print_json(data=active)


@detect.command('list-probes')
@click.option('--days', help='days to look back', default=7, type=int)
@click.pass_obj
@handle_api_error
def list_probes(controller, days):
    """ List all endpoint probes """
    print_json(data=controller.list_probes(days=days))


@detect.command('activity')
@click.option('--days', help='days to look back', default=7, type=int)
@click.pass_obj
@handle_api_error
def describe_activity(controller, days):
    """ View my Detect results """
    raw = controller.describe_activity(days=days)

    build = BuildController(account=controller.account)
    tests = {row['id']: row['rule'] for row in build.list_tests()}

    report = Table()
    report.add_column('timestamp')
    report.add_column('result ID')
    report.add_column('rule')
    report.add_column('test')
    report.add_column('endpoint')
    report.add_column('code', style='magenta')
    report.add_column('status')
    report.add_column('observed')

    for record in raw:
        report.add_row(
            record['date'], 
            record['id'], 
            tests[record['test']], 
            record['test'],
            record['endpoint_id'], 
            str(record['status']),
            Lookup(record['status']).name,
            'yes' if record.get('observed') else '-'
        )

    console = Console()
    console.print(report)


@detect.command('social-stats')
@click.argument('test')
@click.option('--days', help='days to look back', default=30, type=int)
@click.pass_obj
@handle_api_error
def describe_activity(controller, test, days):
    """ Pull social statistics for a specific test """
    print_json(data=controller.stats(ident=test, days=days))


@detect.command('observe')
@click.argument('result')
@click.option('--value', help='Mark 1 for observed', default=1, type=int)
@click.pass_obj
@handle_api_error
def observe(controller, result, value):
    """ Mark a result as observed """
    controller.observe(row_id=result, value=value)
